//***************************************************************************************
//***
//***	Projectname:	Control Interface	
//***	Filename:		C_Interf.c
//***	Shortcut:		CIF
//***	Author:			Ralf Hochhausen
//***	Date:			22.10.2003
//***	Acual Version:	V1.0 (Initial Version)
//***	
//***	Description:	Control interface protokoll	over RS232
//***
//***	File-History:	22.10.2003
//***					V1.0	Initial Version
//***
//***************************************************************************************

//***************************************************************************************
//***
//***	Includes
//***
//***************************************************************************************
#include "Types.h"
#include "GlobalDef.h"
#include "ASCInterface.h"

//***************************************************************************************
//***
//***	Defines
//***
//***************************************************************************************
#define CIF_TIMEOUT_TIME		100			//Timeout after 100ms

//***************************************************************************************
//***
//***	Enumerations
//***
//***************************************************************************************

//***Status Bits
enum
{
	CIF_STATE_OK		=	0x00,		//Interface is ok
	CIF_STATE_ERROR		=	0x01		//Bit0	=>	Error
};

//***Commands
enum
{
	CIF_CMD_NO_COMMAND		= 0x00,
	CIF_CMD_INIT			= 0x55,
	CIF_CMD_READ_BYTE		= 0x01,
	CIF_CMD_WRITE_BYTE		= 0x02,
	CIF_CMD_READ_WORD		= 0x03
};

//***************************************************************************************
//***
//***	Structures
//***
//***************************************************************************************

//***************************************************************************************
//***
//***	Prototypes of statemachine functions
//***
//***************************************************************************************
static void vWaitForInit(void);
static void vWaitForCommand(void);
static void vWaitForAddressHigh(void);
static void vWaitForAddressLow(void);
static void vReadByte(void);
static void vWaitForDataByte(void);
static void vWriteDataByte(void);
static void vReadWord(void);

//***************************************************************************************
//***
//***	Local Variables
//***
//***************************************************************************************
static uchar *CIF_pucAddress;			//Addresspointer to the memory, that has to be
										//written or read
static uchar CIF_ucDataLow;				//Lowbyte of the data (for byte and word access)
static uchar CIF_ucDataHigh;			//Highbyte of the Data
static uchar CIF_ucCtrlCommand;			//Control command, that has to be executed
static uchar CIF_ucStatus;				//Actual status of the interface
										//Bit 0	=>	Error (Reset interface)
										//Bit 1 =>	Byte received
static void (*CIF_fpvActualState)(void);//Functionpointer to the next state

//***************************************************************************************
//***
//***	Local functions
//***
//***************************************************************************************

//***************************************************************************************
//***	Statemachine
//***************************************************************************************

//***************************************************************************************
//***
//***	Function:		void vWaitForInit(void)
//***	
//***	Return Type:	void
//***
//***	Parameters:		none
//***	
//***	Function waits for initialization over serial interface
//***
//***************************************************************************************
static void vWaitForInit(void)
{
	if(ASC_ucIsByteReceived())	//New byte received
	{
		if(ASC_vGetReceivedByte() == CIF_CMD_INIT)
		{
			ASC_vTransmitByte(CIF_CMD_INIT);
			CIF_fpvActualState = vWaitForCommand;
		}
	}
}

//***************************************************************************************
//***
//***	Function:		void vWaitForCommand(void)
//***	
//***	Return Type:	void
//***
//***	Parameters:		none
//***	
//***	Function waits for a control command
//***
//***************************************************************************************
static void vWaitForCommand(void)
{
	uchar ucData_;				//Teporary data Variable
	
	if(ASC_ucIsByteReceived())	//New byte received
	{	
		ucData_ = ASC_vGetReceivedByte();
		if(ucData_ == CIF_CMD_READ_BYTE)			//Valid command received?
		{
				CIF_ucCtrlCommand = ucData_;				//Store command
				ASC_vTransmitByte(CIF_ucCtrlCommand);		//Echo
				CIF_fpvActualState = vWaitForAddressHigh;	//configure next state
		}
	}
}

//***************************************************************************************
//***
//***	Function:		void vWaitForAddressHigh(void)
//***	
//***	Return Type:	void
//***
//***	Parameters:		none
//***	
//***	Function waits for a received byte and writes it into the highbyte
//***	of the address pointer
//***
//***************************************************************************************
static void vWaitForAddressHigh(void)
{
	uchar ucData_;

	CIF_pucAddress = (uchar*)(0x0000);		//reset address pointer

	if(ASC_ucIsByteReceived())	//New byte received
	{
		ucData_ = ASC_vGetReceivedByte();

		//shift received byte into highbyte of the address
		CIF_pucAddress = (uchar*)((uint)ucData_ << 8);		
		ASC_vTransmitByte(ucData_);					//Echo
		CIF_fpvActualState = vWaitForAddressLow;	//configure next state
	}
}

//***************************************************************************************
//***
//***	Function:		void vWaitForAddressLow(void)
//***	
//***	Return Type:	void
//***
//***	Parameters:		none
//***	
//***	Function waits for a received byte and writes it into the lowbyte
//***	of the address pointer
//***
//***************************************************************************************
static void vWaitForAddressLow(void)
{
	uchar ucData_;

	if(ASC_ucIsByteReceived())	//New byte received
	{
		ucData_ = ASC_vGetReceivedByte();

		//write received byte into lowbyte of the address
		CIF_pucAddress = (uchar*)(CIF_pucAddress + (uint)ucData_);		
		ASC_vTransmitByte(ucData_);					//Echo
		switch(CIF_ucCtrlCommand)		//configure next state
		{
			case CIF_CMD_READ_BYTE:
			{
				CIF_fpvActualState = vReadByte;				
			}

			case CIF_CMD_WRITE_BYTE:
			{
				CIF_fpvActualState = vWaitForDataByte;			
			}

			case CIF_CMD_READ_WORD:
			{
				CIF_fpvActualState = vReadWord;
			}
		}
	}
}

//***************************************************************************************
//***
//***	Function:		void vReadByte(void)
//***	
//***	Return Type:	void
//***
//***	Parameters:		none
//***	
//***	Function reads one byte out of the memory place
//***	on which CIF_pucAddress is pointing
//***
//***************************************************************************************
static void vReadByte(void)
{
	CIF_ucDataLow = *CIF_pucAddress;	//Read memory
	ASC_vTransmitByte(CIF_ucDataLow);	//return read memory value
	CIF_fpvActualState = vWaitForCommand;	//configure next state
}

//***************************************************************************************
//***
//***	Function:		void vWaitForDataByte(void)
//***	
//***	Return Type:	void
//***
//***	Parameters:		none
//***	
//***	Function waits for data byte, that has to be written
//***
//***************************************************************************************
static void vWaitForDataByte(void)
{
	uchar ucData_;

	if(ASC_ucIsByteReceived())	//New byte received
	{
		ucData_ = ASC_vGetReceivedByte();

		CIF_ucDataLow = ucData_;		//Store data

		CIF_fpvActualState = vWriteDataByte;
	}
}

//***************************************************************************************
//***
//***	Function:		void vWriteDataByte(void)
//***	
//***	Return Type:	void
//***
//***	Parameters:		none
//***	
//***	Function writes received data byte into memory (RAM)
//***
//***************************************************************************************
static void vWriteDataByte(void)
{
	uchar ucData_;

	*CIF_pucAddress = CIF_ucDataLow;		//Write data byte into RAM
	ucData_ = *CIF_pucAddress;				//Verify data byte
	ASC_vTransmitByte(ucData_);				//Echo
	
	CIF_fpvActualState = vWaitForCommand;
}

//***************************************************************************************
//***
//***	Function:		void vReadWord(void)
//***	
//***	Return Type:	void
//***
//***	Parameters:		none
//***	
//***	Function reads one word out of the memory place
//***	on which CIF_pucAddress is pointing
//***
//***************************************************************************************
static void vReadWord(void)
{
	CIF_ucDataHigh = *(CIF_pucAddress +1);	//Read memory high byte
	CIF_ucDataLow  = *CIF_pucAddress;		//Read memory low byte
	ASC_vTransmitByte(CIF_ucDataHigh);		//return read memory value high byte
	ASC_vTransmitByte(CIF_ucDataLow);		//return read memory value low byte
	CIF_fpvActualState = vWaitForCommand;	//configure next state
}

//***************************************************************************************
//***
//***	Global Functions
//***
//***************************************************************************************

//***************************************************************************************
//***
//***	Function:		void CIF_vHandleTask(void)
//***	
//***	Return Type:	void
//***
//***	Parameters:		none
//***	
//***	Task Handler of the Module
//***
//***************************************************************************************
void CIF_vHandleTask(void)
{
	if(CIF_ucStatus & CIF_STATE_ERROR)			//Interface in error State?
	{
		CIF_fpvActualState = vWaitForInit;		//Reset statemachine
	}

	CIF_fpvActualState();				//Call state function
}

//***************************************************************************************
//***
//***	Function:		void CIF_vInit(void)
//***	
//***	Return Type:	void
//***
//***	Parameters:		none
//***	
//***	Intialization of the control interface
//***
//***************************************************************************************
void CIF_vInit(void)
{
	CIF_pucAddress = NULL;
	CIF_ucDataLow = 0x00;
	CIF_ucDataHigh = 0x00;
	CIF_ucCtrlCommand = (uchar)CIF_CMD_NO_COMMAND;	//No command after startup 
	CIF_ucStatus = CIF_STATE_OK;					//Control State is OK after startup
	CIF_fpvActualState = vWaitForInit;				//Wait for initialization first
}
